;; -*- lexical-binding: t -*-

(eval-when-compile (require 'cl-lib))

(defvar test--mb-hints-timer nil)

(defun test-trigger-mb-hints ()
  (if (not (timerp test--mb-hints-timer))
      (setq test--mb-hints-timer (run-at-time 2 nil #'test--display-mb-hints))
    (timer-set-time test--mb-hints-timer (time-add (current-time) .5))
    (timer-activate test--mb-hints-timer)))

(defun test-setup-mb ()
  (when (timerp test--mb-hints-timer) (cancel-timer test--mb-hints-timer))
  (setq test--mb-hints-timer nil)
  (add-hook 'post-command-hook #'test-trigger-mb-hints t t))

(defun test--display-mb-hints ()
  (when (minibufferp)
    (while-no-input
      (let ((ol (make-overlay (point-max) (point-max) nil t t)))
        (unwind-protect
            (cl-flet ((display-message
                       (lambda (message &rest args)
                         (setq message
                               (propertize (apply #'format message args)
                                           'face 'shadow))
                         (put-text-property 0 1 'cursor t message)
                         (overlay-put ol 'after-string message)
                         (redisplay))))
              (display-message "     [A useful hint]")
              (sit-for 5))
          (delete-overlay ol))))
    (when quit-flag
      ;; When `quit-flag' is bound here, it had been set by `while-no-input'
      ;; meaning the user explicitly quit.  This means we must:
      (funcall (key-binding [(control ?g)])))))

;; f12 to start test:
(global-set-key
 [f12]
 (defun my-test ()
   (interactive)
   (minibuffer-with-setup-hook #'test-setup-mb
     (read-string "Enter a string: "))))
